@@ -10,7 +10,7 @@
 
 import re
 
-__version__ = '0.3.0'
+__version__ = '0.3.1'
 __author__ = 'Hsiaoming Yang <me@lepture.com>'
 __all__ = [
     'BlockGrammar', 'BlockLexer',
@@ -459,6 +459,15 @@ class InlineGrammar(object):
     footnote = re.compile(r'^\[\^([^\]]+)\]')
     text = re.compile(r'^[\s\S]+?(?=[\\<!\[_*`~]|https?://| {2,}\n|$)')
 
+    def hard_wrap(self):
+        """Grammar for hard wrap linebreak. You don't need to add two
+        spaces at the end of a line.
+        """
+        self.linebreak = re.compile(r'^ *\n(?!\s*$)')
+        self.text = re.compile(
+            r'^[\s\S]+?(?=[\\<!\[_*`~]|https?://| *\n|$)'
+        )
+
 
 class InlineLexer(object):
     """Inline level lexer for inline grammars."""
@@ -480,12 +489,9 @@ class InlineLexer(object):
 
         if not rules:
             rules = InlineGrammar()
-            if self.options.get('hard_wrap'):
-                # {2,} -> *
-                rules.linebreak = re.compile(r'^ *\n(?!\s*$)')
-                rules.text = re.compile(
-                    r'^[\s\S]+?(?=[\\<!\[_*`~]|https?://| *\n|$)'
-                )
+
+        if self.options.get('hard_wrap'):
+            rules.hard_wrap()
 
         self.rules = rules
 
@@ -622,7 +628,8 @@ class InlineLexer(object):
         return self.renderer.emphasis(text)
 
     def output_code(self, m):
-        text = escape(m.group(2), smart_amp=False)
+        text = m.group(2)
+        text = escape(text.rstrip(), smart_amp=False)
         return self.renderer.codespan(text)
 
     def output_linebreak(self, m):
@@ -650,12 +657,11 @@ class Renderer(object):
         :param code: text content of the code block.
         :param lang: language of the given code.
         """
-        code = code.rstrip()
         if not lang:
             code = escape(code, smart_amp=False)
-            return '<pre><code>%s\n</code></pre>\n' % code
+            return '<pre><code>%s</code></pre>\n' % code
         code = escape(code, quote=True, smart_amp=False)
-        return '<pre><code class="lang-%s">%s\n</code>\n</pre>' % (lang, code)
+        return '<pre><code class="lang-%s">%s</code></pre>\n' % (lang, code)
 
     def block_quote(self, text):
         """Rendering <blockquote> with the given text.
@@ -888,6 +894,13 @@ class Markdown(object):
         self.renderer = renderer
         self.options = kwargs
 
+        if inline and callable(inline):
+            # assume it is a class
+            inline = inline(renderer, **kwargs)
+        if block and callable(block):
+            # assume it is a class
+            block = block(**kwargs)
+
         self.inline = inline or InlineLexer(renderer, **kwargs)
         self.block = block or BlockLexer(**kwargs)
 
